let data;
let AXIS_INSET;
let AXIS_WIDTH;
let AXIS_HEIGHT;
let smallestTick;
let largestTick;
let numTicks;
let tickSpace;

function preload() {
  data = loadTable('https://gist.githubusercontent.com/colmpat/1a46fcfd6e6c89b2a8e83ebeeceaf58f/raw/eb3f3cb65237d7195e6f6dfd6b2e412b917a6312/Europe-Power-Sector-Dot-Plot.csv', 'csv', 'header');
}

function setup() {
  createCanvas(800, 500);
  angleMode(DEGREES);
  rectMode(CENTER);
  X_INSET_LEFT = 155;
  X_INSET_RIGHT = 20;
  Y_INSET = 70;
  AXIS_WIDTH = width - X_INSET_LEFT - X_INSET_RIGHT;
  AXIS_HEIGHT = height - 2 * Y_INSET;

  smallestTick = floor(data.getColumn(1).at(-1) / 5.0) * 5.0;
  largestTick = ceil(data.getColumn(1).at(0) / 5.0) * 5.0;
  numTicks = (largestTick - smallestTick) / 5.0;
  tickSpace = AXIS_WIDTH / (numTicks + 1);
}

function draw() {
  background(250,250,255);

  title("Avg. Share of Production per Country in 2020 for Renewables", X_INSET_LEFT);
  drawTicks(X_INSET_LEFT, Y_INSET);
  drawAxis(X_INSET_LEFT, X_INSET_RIGHT, Y_INSET);
  labelAxis("Avg. Share of Production", "", 20, 5, 4);
  drawDots(X_INSET_LEFT, Y_INSET, 20, 15);

  noLoop();
}

function title(title, inset) {
  textSize(23); fill(37,48,49);
  text(title, inset + 10, 40);
}

function drawAxis(xInsetLeft, xInsetRight, yInset) {
  stroke(37,48,49); strokeWeight(3);
  //top left to bottom left
  line(xInsetLeft, yInset - 10, xInsetLeft, height - yInset);
  //bottom left to bottom right
  line(xInsetLeft, height - yInset, width - xInsetRight, height - yInset);
}

function labelAxis(xLabel, yLabel, inset, numTicksX, numTicksY) {
  textAlign(CENTER, CENTER); textSize(18); noStroke();
  //label x axis
  let xCentered = ((width - X_INSET_LEFT - X_INSET_RIGHT) / 2.0) + X_INSET_LEFT;
  text(xLabel, xCentered, height - inset);

  //translate and rotate vertically
  translate(inset, height / 2); rotate(-90);
  //label y axis at the origin of translated grid
  text(yLabel, inset, 0);
  resetMatrix();
}

function drawTicks(xInset, yInset) {
  let runningX = xInset + tickSpace / 2.0;

  for(let i = smallestTick; i <= largestTick; i += 5) {
    strokeWeight(2); stroke(132,130,143);
    line(runningX, yInset - 10, runningX, height - yInset);
    stroke(37,48,49); strokeWeight(3);
    line(runningX, height - yInset, runningX, height - yInset + 10);
    noStroke(); textSize(20); textAlign(CENTER);
    text(i, runningX, height - yInset + 25);
    runningX += tickSpace;
  }
}

function drawDots(xInset, yInset, spacing, d) {
  let runningY = height - yInset - spacing - d / 2.0;
  let numRows = data.getRowCount();
  for(let i = numRows - 1; i >= 0; i--) {
    let currentRow = data.getRow(i);

    textSize(20); textAlign(RIGHT); fill(37,48,49);
    text(currentRow.get(0), xInset - 8, runningY);

    fill(88,188,130);
    circle(getX(currentRow.get(1)), runningY, d);

    runningY -= d + spacing;
  }

}

function getX(shareOfProduction) {
  return (AXIS_WIDTH - tickSpace) * (shareOfProduction - smallestTick) / (largestTick - smallestTick) + X_INSET_LEFT + tickSpace / 2.0; //ratio: original - smallestTick / largestTick - smallestTick = new / AXIS_WIDTH
}
